<?php

namespace Drupal\commerce_product_type_fees;

use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\commerce_order\OrderProcessorInterface;
use Drupal\commerce_price\Price;
use Drupal\commerce_product\Entity\Product;
use Drupal\commerce_order\Adjustment;

/**
 * Applies custom fees to orders during the order refresh process.
 */
class FeesOrderProcessor implements OrderProcessorInterface {

  /**
   * {@inheritdoc}
   */
  public function process(OrderInterface $order) {
    $order->setAdjustments([]);
    $hasCustomFee = FALSE;
    foreach ($order->getItems() as $orderItem) {
      $product_variation = $orderItem->getPurchasedEntity();
      if (!empty($product_variation)) {
        $product_id = $product_variation->get('product_id')
          ->getValue()[0]['target_id'];
        $product = Product::load($product_id);
        $product_type = $product->get('type')->getValue()[0]['target_id'];
        if ($product_type) {
          $config = \Drupal::config('commerce_product_type_fees.settings');
          $custom_fees = $config->get($product_type . '_fees');
          if (isset($custom_fees) && !empty($custom_fees)) {
            // Fees is added into backend for this product types.
            $hasCustomFee = TRUE;
          }
          break;
        }
      }
    }

    if ($hasCustomFee) {
      // Apply all fees for that product type on order subtotal.
      $subtotal = $order->getSubtotalPrice();
      // Remove currency symbol.
      $subtotalNumber = array_shift(explode(' ', $subtotal));
      foreach ($custom_fees as $fee) {
        if (isset($fee['percentage']) && !empty($fee['percentage'])) {
          $percentage = $fee['percentage'] / 100;
          $new_adjustment = sprintf("%.2f", $subtotalNumber * $percentage);
          // Apply custom adjustment.
          $this->addFee($order, 'custom_fee', $fee['fee']['label'], new Price($new_adjustment, 'USD'));
        }
      }
    }
  }

  /**
   * Function to add an adjustment.
   *
   * @param \Drupal\commerce_order\Entity\OrderInterface $order
   *   The order.
   * @param mixed $type
   *   The adjustment type.
   * @param mixed $label
   *   The adjustment label.
   * @param \Drupal\commerce_price\Price $price
   *   The adjustment price.
   */
  protected function addFee(OrderInterface $order, $type, $label, Price $price) {
    $order->addAdjustment(new Adjustment([
      'type' => $type,
      'label' => $label,
      'amount' => $price,
    ]));
  }

}
